from django.test import TestCase
from django.conf import settings
from django.core.management import call_command

from shopify_auth.models import AbstractShopUser


class AuthAppShopUser(AbstractShopUser):
    class Meta:
        app_label = 'shopify_auth'


class ViewsTestCase(TestCase):

    def setUp(self):
        settings.AUTH_USER_MODEL = 'shopify_auth.AuthAppShopUser'

    def test_create_super_user(self):
        call_command('createsuperuser', '--noinput',  myshopify_domain='myshop.shopify.com')



from django.conf import settings


def shopify_auth(request):
    return {
        'SHOPIFY_APP_NAME': getattr(settings, 'SHOPIFY_APP_NAME'),
        'SHOPIFY_APP_API_KEY': getattr(settings, 'SHOPIFY_APP_API_KEY'),
        'SHOPIFY_APP_IS_EMBEDDED': getattr(settings, 'SHOPIFY_APP_IS_EMBEDDED'),
        'SHOPIFY_APP_DEV_MODE': getattr(settings, 'SHOPIFY_APP_DEV_MODE'),
    }



from django.test import TestCase
from ..helpers import add_query_parameters_to_url


class HelpersTestCase(TestCase):

    def test_add_query_parameters_to_url(self):
        """
        Test add_query_parameters_to_url helper.
        """
        # Define a list of test cases as triples in the format (url, query_parameters, expected_url)
        test_cases = [
            ('http://example.com', {'page': 2}, 'http://example.com?page=2'),
            ('http://example.com?size=xl', {'page': 5}, 'http://example.com?page=5&size=xl'),
            ('http://example.com?size=xl&page=1', {'page': 5}, 'http://example.com?page=5&size=xl'),
        ]

        # Run each test case through the helper and check the expected output.
        for url, query_parameters, expected_url in test_cases:
            self.assertEqual(add_query_parameters_to_url(url, query_parameters), expected_url)



from django.test import TestCase, Client
from django.conf import settings


class ViewsTestCase(TestCase):

    def setUp(self):
        self.client = Client()

    def tearDown(self):
        self.client = None

    def test_login_view(self):
        """
        Test the login view loads when we're an anonymous user.
        """
        response = self.client.get('/')
        self.assertEqual(response.status_code, 200)

    def test_authenticate_view(self):
        """
        Test the authenticate view renders correctly with a shop param.
        """
        response = self.client.get('/authenticate/')
        self.assertEqual(response.status_code, 302)
        response = self.client.get('/authenticate/', follow=True)
        self.assertEqual(response.status_code, 404)
        response = self.client.post('/authenticate/')
        self.assertEqual(response.status_code, 302)

        response = self.client.get('/?shop=test.myshopify.com')
        self.assertContains(response, 'window.top.location.href = "https://test.myshopify.com/admin/oauth/authorize')

        # Dev mode so token does not need to be valid
        settings.SHOPIFY_APP_DEV_MODE = True
        response = self.client.get('/authenticate/?shop=test.myshopify.com')
        self.assertEqual(response.status_code, 302)
        self.assertGreater(int(self.client.session['_auth_user_id']), 0)
        self.assertEqual(self.client.session['_auth_user_backend'], 'shopify_auth.backends.ShopUserBackend')
        self.assertIsNot(self.client.session['_auth_user_hash'], None)





import shopify

from django.contrib.auth.models import AbstractBaseUser, BaseUserManager
from django.db import models
from django.utils.encoding import python_2_unicode_compatible


class ShopUserManager(BaseUserManager):

    def create_user(self, myshopify_domain, password=None):
        """
        Creates and saves a ShopUser with the given domain and password.
        """
        if not myshopify_domain:
            raise ValueError('ShopUsers must have a myshopify domain')

        user = self.model(myshopify_domain=myshopify_domain)

        # Never want to be able to log on externally.
        # Authentication will be taken care of by Shopify OAuth.
        user.set_unusable_password()
        user.save(using=self._db)
        return user

    def create_superuser(self, myshopify_domain, password):
        """
        Creates and saves a ShopUser with the given domains and password.
        """
        return self.create_user(myshopify_domain, password)


@python_2_unicode_compatible
class AbstractShopUser(AbstractBaseUser):
    myshopify_domain = models.CharField(max_length=255, unique=True, editable=False)
    token = models.CharField(max_length=32, editable=False, default='00000000000000000000000000000000')

    objects = ShopUserManager()

    USERNAME_FIELD = 'myshopify_domain'

    @property
    def session(self):
        return shopify.Session.temp(self.myshopify_domain, self.token)

    def get_full_name(self):
        return self.myshopify_domain

    def get_short_name(self):
        return self.myshopify_domain

    def __str__(self):
        return self.get_full_name()

    class Meta:
        abstract = True



from django.contrib.auth.backends import RemoteUserBackend


class ShopUserBackend(RemoteUserBackend):
    def authenticate(self, request=None, myshopify_domain=None, token=None, **kwargs):
        if not myshopify_domain or not token or not request:
            return

        try:
            user = super(ShopUserBackend, self).authenticate(request=request, remote_user=myshopify_domain)
        except TypeError:
            #  Django < 1.11 does not have request as a mandatory parameter for RemoteUserBackend
            user = super(ShopUserBackend, self).authenticate(remote_user=myshopify_domain)

        if not user:
            return

        user.token = token
        user.save(update_fields=['token'])
        return user
